%     Ventricular action potential model ORdmJ
%     Copyright (C) 2020 Jordi Llopis. Contact: jorllolo@etsii.upv.es
% 
%     This program is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <https://www.gnu.org/licenses/>

function [ block ] = calculateBlock(parameters)
% This function returns a vector with the blocking percentage for each current [INa INaL
% Ito ICaL IKr IKs IK1]
% parameters is an structure array containing the following fields:
%     D: drug contentration
%     ICNa: IC 50 for INa channel
%     hNa: Hill coefficient for INa channel
%     ICNaL: IC 50 for INaL channel
%     hNaL: Hill coefficient for INaL channel
%     ICto: IC 50 for Ito channel
%     hto: Hill coefficient for Ito channel
%     ICCaL: IC 50 for ICaL channel
%     hCaL: Hill coefficient for ICaL channel
%     ICKr: IC 50 for IKr channel
%     hKr: Hill coefficient for IKr channel
%     ICKs: IC 50 for IKs channel
%     hKs: Hill coefficient for IKs channel
%     ICK1: IC 50 for IK1 channel
%     hK1: Hill coefficient for IK1 channel
% If an IC 50 for a given current is not provided, it is assumed that the
% current is not blocked
% If a Hill coefficient is not defined, it is assumed to be equal to 1
block=zeros(7,1);
if (isfield(parameters, 'D')) && ~isempty(parameters.D)
    D = parameters.D; 
else
    error('D field is empty. A concentration must be assigned'); 
end
if isfield(parameters,'ICNa')&& ~isempty(parameters.ICNa)
    if isfield(parameters, 'hNa')&& ~isempty(parameters.hNa)
        hNa= parameters.hNa;
    else
        hNa=1;
    end
    ICNa=parameters.ICNa;
    block(1)=1-1/(1+(D/ICNa)^hNa);
end
if isfield(parameters,'ICNaL')&& ~isempty(parameters.ICNaL)
    if isfield(parameters, 'hNaL')&& ~isempty(parameters.hNaL)
        hNaL= parameters.hNaL;
    else
        hNaL=1;
    end
    ICNaL=parameters.ICNaL;
    block(2)=1-1/(1+(D/ICNaL)^hNaL);
end
if isfield(parameters,'ICto')&& ~isempty(parameters.ICto)
    if isfield(parameters, 'hto')&& ~isempty(parameters.hto)
        hto= parameters.hto;
    else
        hto=1;
    end
    ICto=parameters.ICto;
    block(3)=1-1/(1+(D/ICto)^hto);
end
if isfield(parameters,'ICCaL')&& ~isempty(parameters.ICCaL)
    if isfield(parameters, 'hCaL')&& ~isempty(parameters.hCaL)
        hCaL= parameters.hCaL;
    else
        hCaL=1;
    end
    ICCaL=parameters.ICCaL;
    block(4)=1-1/(1+(D/ICCaL)^hCaL);
end
if isfield(parameters,'ICKr')&& ~isempty(parameters.ICKr)
    if isfield(parameters, 'hKr')&& ~isempty(parameters.hKr)
        hKr= parameters.hKr;
    else
        hKr=1;
    end
    ICKr=parameters.ICKr;
    block(5)=1-1/(1+(D/ICKr)^hKr);
end
if isfield(parameters,'ICKs')&& ~isempty(parameters.ICKs)
    if isfield(parameters, 'hKs')&& ~isempty(parameters.hKs)
        hKs= parameters.hKs;
    else
        hKs=1;
    end
    ICKs=parameters.ICKs;
    block(6)=1-1/(1+(D/ICKs)^hKs);
end
if isfield(parameters,'ICK1')&& ~isempty(parameters.ICK1)
    if isfield(parameters, 'hK1')&& ~isempty(parameters.hK1)
        hK1= parameters.hK1;
    else
        hK1=1;
    end
    ICK1=parameters.ICK1;
    block(7)=1-1/(1+(D/ICK1)^hK1);
end
end