%     Ventricular action potential model ORdmJ
%     Copyright (C) 2020 Jordi Llopis. Contact: jorllolo@etsii.upv.es
% 
%     This program is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <https://www.gnu.org/licenses/>

function [Tx, Tx_ev, output] = calculate_Tx(parameters, time_initial, v_initial)
% This function iteratively searches for the concentration that produces a 
% 10% prolongation of the APD90. It is assumed that concentration stored in
% parameters.D is the EFTPC
% INPUTS:
% parameters: a structure array containing numbers of beats of the
% simulation, cycle length, initial conditions, cell type, drug
% concentration...
% time_initial: time vector of the simulation under initial drug
% concentration (i.e. EFTPC)
% v_initial: membrane voltage values at corresponding time point under
% initial drug contrentation (i.e. EFTPC)
% OUTPUTS:
% Tx: value of the index Tx
% Tx_ev: vector containing the values of [] / EFTPC for the different
% concentrations tested from initial drug concentration (assumed to be
% EFTPC)
% output: structure array containing time vector, state variable and 
% currents correspondig to the last beat simulated under
% concentration that prolongs APD90 a 10%


[APD90,~,~,~,~,~]=APD(time_initial,v_initial,90); % Calculate APD90 at EFTPC
APD_control= 263.3883; % APDControl for celltype='endo', beats= 1000 beats, CL=1000 ms
delta1=(APD90 - APD_control)/APD_control*100;

D(1)=parameters.D;
EFTPC=D(1);

it=0; i=1;
if delta1>10
    delta=delta1;
    while (delta>10) && it < 100
        deltas(2)= delta;
        i=i+1;
        D(i)=D(i-1)*10^(-0.1);
        parameters.D=D(i);
        
        [time, X, IsJs]=modelRunner(parameters);
        
        [APD90,~,~,~,~,~]=APD(time,X(:,1),90);
        
        delta=(APD90-APD_control)/APD_control*100;
        deltas(1) = delta;
        it=it+1;
        disp(['Reducing initial [] ' num2str(it) 'times'])
    end
    if abs(10-deltas (1)) < abs(deltas(2)-10)
        Tx=D(i)/EFTPC;
        output.time=time; output.X=X; output.IsJs=IsJs;
    else
        Tx=D(i-1)/EFTPC;
        parameters.D=D(i-1);
        [time, X, IsJs]=modelRunner(parameters);
        output.time=time; output.X=X; output.IsJs=IsJs;
    end
end
if delta1<10
    delta=delta1;
    while (delta<10) && it < 100
        deltas (2)= delta;
        i=i+1;
        D(i)=D(i-1)*10^(+0.1);
        parameters.D=D(i);
        
        [time,X, IsJs]=modelRunner(parameters);
        [APD90,~,~,~,~,~]=APD(time,X(:,1),90);
        
        delta=(APD90-APD_control)/APD_control*100;
        deltas(1) = delta;
        it=it+1;
        disp(['Increasing initial [] ' num2str(it) 'times'])
        
    end
    if abs(10-deltas (1)) < abs(deltas(2)-10)
        Tx=D(i)/EFTPC;
        output.time=time; output.X=X; output.IsJs=IsJs;
    else
        Tx=D(i-1)/EFTPC;
        parameters.D=D(i-1);
        [time, X, IsJs]=modelRunner(parameters);
        output.time=time; output.X=X; output.IsJs=IsJs;
    end
end
Tx_ev=D./EFTPC;
disp(['Finished Tx = ' num2str(Tx)])
end
